## parse annotated pepXML's into a more managable format
## Author: Chris, Stephan

import array
from datetime import datetime, date, time
import sys
import os
import shutil
#import io
import argparse
import random
import math
from copy import deepcopy
import glob
import re

import numpy as np
#import matplotlib.pyplot as plt
#import matplotlib.mlab as mlab

#from xml.etree.ElementTree import ElementTree
import xml.etree.ElementTree as ET

## extracts the data
def run(infile_name, outfile_name, xpress_format):
  if (not infile_name.endswith(".pepXML")):
    raise Exception ("infile has no .pepXML suffix");
  if (outfile_name==None):
    outfile_name = infile_name.replace(".pepXML", "_parsed.result")
  print "infile: ", infile_name
  print "outfile: ", outfile_name
  
  data_out = []
  
  if (not xpress_format):  
    # parse XML
    root = ET.ElementTree()
    root.parse(infile_name)
    rootElement = root.getroot()
  
    iter = rootElement.getiterator()
    
    for element in iter:
      #First the element tag name
      if element.tag == '{http://regis-web.systemsbiology.net/pepXML}spectrum_query':
        peptide = ""
        ratio = ""

        charge =  element.get('assumed_charge')
        children = element.getchildren()
        search_hits = children[0].getchildren()
        # find search hit
        for hit in search_hits:
          if hit.tag == '{http://regis-web.systemsbiology.net/pepXML}search_hit':        
            peptide = hit.get('peptide')
            hit_children = hit.getchildren()
            
            for hit_child in hit_children:
              if hit_child.tag == '{http://regis-web.systemsbiology.net/pepXML}analysis_result' and hit_child.get('analysis') == 'asapratio':
                # we got the ratio
                asap_results = hit_child.getchildren()
                for asap_result in asap_results:
                  if asap_result.tag == '{http://regis-web.systemsbiology.net/pepXML}asapratio_result'
                    ratio = asap_result.get('mean')
        if charge != "" and peptide != "" and ratio != "":
          # found a complete analysis
          d = [ peptide , charge, ratio]
          data_out.append( d )  
  else:
    # parse XML
    root = ET.ElementTree()
    root.parse(infile_name)
    rootElement = root.getroot()
  
    iter = rootElement.getiterator()
    
    for element in iter:
      #First the element tag name
      if element.tag == '{http://regis-web.systemsbiology.net/pepXML}spectrum_query':
        peptide = ""
        ratio = ""

        charge =  element.get('assumed_charge')
        children = element.getchildren()
        search_hits = children[0].getchildren()
        # find search hit
        for hit in search_hits:
          if hit.tag == '{http://regis-web.systemsbiology.net/pepXML}search_hit':        
            peptide = hit.get('peptide')
            hit_children = hit.getchildren()
            
            for hit_child in hit_children:
              if hit_child.tag == '{http://regis-web.systemsbiology.net/pepXML}analysis_result' and hit_child.get('analysis') == 'xpress':
                # we got the ratio
                xpress_tag = hit_child.getchildren()
                ratio = xpress_tag[0].get("decimal_ratio")
        if charge != "" and peptide != "" and ratio != "":
          # found a complete analysis
          d = [ peptide , charge, ratio]
          data_out.append( d )
      
  print len(data_out), " matches found"
  
  # just for debugging
  if len(data_out) == 0:
    return
  
  ## write result
  f = open(outfile_name, 'w')
  for hit in data_out:
    f.write('\t'.join(hit) + "\n")
  f.close()
  
  ## prelim plotting --- remove if this becomes annoying
#  fig = plt.figure()
#  ax = fig.add_subplot(111)

#  arr = np.array(data_out)
  # the histogram of the data
#  data = np.log( np.array(arr[:,2], dtype=np.float32) )
#  data = data[~np.isnan(data)]
#  data = data[~np.isinf(data)]
#  n, bins, patches = ax.hist(data, 1000)
  
#  ax.set_xlabel('ratio')
#  ax.set_ylabel('count')
#  #ax.set_title(r'$\mathrm{Histogram\ of\ IQ:}\ \mu=100,\ \sigma=15$')
#  ax.set_xlim(0,1)
#  ax.grid(True)

#  plt.show()
  
## // end run()  

if __name__ == "__main__":

    parser = argparse.ArgumentParser(description='Parse Annotated pepXML (ASAPRatio or XPress)',
             formatter_class=argparse.RawDescriptionHelpFormatter,
				)
    parser.add_argument('infile', action='store', help='Annotated pepXML (by ASAPRatio or XPress')
    parser.add_argument('--outfile', action='store', help='name of parsed CSV file (defaults to "<infile_prefix>_parsed.result")')
    parser.add_argument('--xpress', dest='xpress_format', action='store_true', help='Set to search for XPress results instead of ASAPRatio')
    
    args = parser.parse_args()
    
    run( args.infile, args.outfile, args.xpress_format )